@@ -17,14 +17,33 @@
 import re
 
 SUPPORTED_FORMATS = {
-    'markdown':         ['.mdown', '.markdown', '.markdn', '.md'],
+    'markdown':         ['.mdown', '.markdown', '.markdn', '.md', '.mdn'],
     'restructuredtext': ['.rst', '.rest'],
+    'textile':          ['.textile'],
 }
 
 
 class Parser(object):
-    def __init__(self, extension, encoding='utf8'):
-        """Configures this parser"""
+    """This class generates the HTML code depending on which syntax is used in
+       the souce document.
+
+       The Parser currently supports both Markdown and restructuredText
+       syntaxes.
+    """
+    RST_REPLACEMENTS = [
+            (r'<div.*?>', r'', re.UNICODE),
+            (r'</div>', r'', re.UNICODE),
+            (r'<p class="system-message-\w+">.*?</p>', r'', re.UNICODE),
+            (r'Document or section may not begin with a transition\.',
+             r'', re.UNICODE),
+            (r'<h(\d+?).*?>', r'<h\1>', re.DOTALL | re.UNICODE),
+            (r'<hr.*?>\n', r'<hr />\n', re.DOTALL | re.UNICODE),
+    ]
+    md_extensions = ''
+
+    def __init__(self, extension, encoding='utf8', md_extensions=''):
+        """Configures this parser.
+        """
         self.encoding = encoding
         self.format = None
         for supp_format, supp_extensions in SUPPORTED_FORMATS.items():
@@ -33,33 +52,40 @@ class Parser(object):
                     self.format = supp_format
         if not self.format:
             raise NotImplementedError(u"Unsupported format %s" % extension)
+        if md_extensions:
+            exts = (value.strip() for value in md_extensions.split(','))
+            self.md_extensions = filter(None, exts)
 
     def parse(self, text):
-        """Parses and renders a text as HTML regarding current format"""
+        """Parses and renders a text as HTML regarding current format.
+        """
         if self.format == 'markdown':
             try:
                 import markdown
             except ImportError:
                 raise RuntimeError(u"Looks like markdown is not installed")
 
-            return markdown.markdown(text)
+            if text.startswith(u'\ufeff'):  # check for unicode BOM
+              text = text[1:]
+
+            return markdown.markdown(text, self.md_extensions)
         elif self.format == 'restructuredtext':
             try:
-                from rst import html_parts, html_body
+                from rst import html_body
             except ImportError:
                 raise RuntimeError(u"Looks like docutils are not installed")
-
             html = html_body(text, input_encoding=self.encoding)
-            html = re.sub(r'<div.*?>\n', r'', html, re.DOTALL | re.UNICODE)
-            html = re.sub(r'</div>\n', r'', html, re.DOTALL | re.UNICODE)
-            html = re.sub(r'<p class="sys.+\n.+ion\.', r'', html,
-                          re.DOTALL | re.UNICODE) # Pretty hackish
-            html = re.sub(r'<h1 class="title">', r'<h1>', html,
-                          re.DOTALL | re.UNICODE)
-            html = re.sub(r'<hr class=".*?" />\n', r'<hr />\n', html,
-                          re.DOTALL | re.UNICODE)
+            # RST generates pretty much markup to be removed in our case
+            for (pattern, replacement, mode) in self.RST_REPLACEMENTS:
+                html = re.sub(pattern, replacement, html, 0, mode)
+            return html.strip()
+        elif self.format == 'textile':
+            try:
+                import textile
+            except ImportError:
+                raise RuntimeError(u"Looks like textile is not installed")
 
-            return html
+            return textile.textile(text, encoding=self.encoding)
         else:
             raise NotImplementedError(u"Unsupported format %s, cannot parse"
                                       % self.format)
